From R.Marek@sh.cvut.cz Wed Jul 27 04:54:06 2005
Date: 27 Jul 2005 11:50:18 -0000
Message-ID: <20050727115018.9241.qmail@desitka.sh.cvut.cz>
From: R.Marek@sh.cvut.cz
To: greg@kroah.com
Subject: I2C: W83792D add hwmon class register 2/3
Cc: <DZShen@Winbond.com.tw>

This patch adds registration of hwmon class. Tested with help of i2c-stub.

Signed-off-by: Rudolf Marek <r.marek@sh.cvut.cz>
Signed-off-by: Greg Kroah-Hartman <gregkh@suse.de>


---
 drivers/hwmon/w83792d.c |   33 +++++++++++++++++++++++++++------
 1 files changed, 27 insertions(+), 6 deletions(-)

--- gregkh-2.6.orig/drivers/hwmon/w83792d.c	2005-07-27 23:03:52.000000000 -0700
+++ gregkh-2.6/drivers/hwmon/w83792d.c	2005-07-27 23:03:54.000000000 -0700
@@ -42,7 +42,9 @@
 #include <linux/i2c.h>
 #include <linux/i2c-sensor.h>
 #include <linux/i2c-vid.h>
+#include <linux/hwmon.h>
 #include <linux/hwmon-sysfs.h>
+#include <linux/err.h>
 
 /* Addresses to scan */
 static unsigned short normal_i2c[] = { 0x2c, 0x2d, 0x2e, 0x2f, I2C_CLIENT_END };
@@ -267,6 +269,7 @@
 
 struct w83792d_data {
 	struct i2c_client client;
+	struct class_device *class_dev;
 	struct semaphore lock;
 	enum chips type;
 
@@ -1289,6 +1292,11 @@
 	}
 
 	/* Register sysfs hooks */
+	data->class_dev = hwmon_device_register(&new_client->dev);
+	if (IS_ERR(data->class_dev)) {
+		err = PTR_ERR(data->class_dev);
+		goto ERROR3;
+	}
 	device_create_file_in(new_client, 0);
 	device_create_file_in(new_client, 1);
 	device_create_file_in(new_client, 2);
@@ -1361,6 +1369,15 @@
 
 	return 0;
 
+ERROR3:
+	if (data->lm75[0] != NULL) {
+		i2c_detach_client(data->lm75[0]);
+		kfree(data->lm75[0]);
+	}
+	if (data->lm75[1] != NULL) {
+		i2c_detach_client(data->lm75[1]);
+		kfree(data->lm75[1]);
+	}
 ERROR2:
 	i2c_detach_client(new_client);
 ERROR1:
@@ -1372,21 +1389,25 @@
 static int
 w83792d_detach_client(struct i2c_client *client)
 {
+	struct w83792d_data *data = i2c_get_clientdata(client);
 	int err;
 
+	/* main client */
+	if (data)
+		hwmon_device_unregister(data->class_dev);
+
 	if ((err = i2c_detach_client(client))) {
 		dev_err(&client->dev,
 			"Client deregistration failed, client not detached.\n");
 		return err;
 	}
 
-	if (i2c_get_clientdata(client)==NULL) {
-		/* subclients */
+	/* main client */
+	if (data)
+		kfree(data);
+	/* subclient */
+	else
 		kfree(client);
-	} else {
-		/* main client */
-		kfree(i2c_get_clientdata(client));
-	}
 
 	return 0;
 }
